Package: ECoL
Type: Package
Version: 0.4.4
Date: 2026-05-19
Title: Complexity Measures for Supervised Problems
Description: Provides measures to characterize the complexity of classification 
    and regression problems based on aspects that quantify the linearity of the 
    data, the presence of informative feature, the sparsity and dimensionality 
    of the datasets. This package provides bug fixes, generalizations and 
    implementations of many state of the art measures. The measures are 
    described in the papers: Lorena et al. (2019) <doi:10.1145/3347711> and 
    Lorena et al. (2018) <doi:10.1007/s10994-017-5681-1>.
Authors@R: c(person(given = "Luis",
                   family = "Garcia",
                   role = c("aut", "cre"),
                   email = "luis.garcia@unb.br"),
            person(given = "Ana",
                   family = "Lorena",
                   role = c("aut", "ctb")),
            person(given = "Marcilio",
                   family = "Souto",
                   role = c("aut", "ctb")),
            person(given = c("Tin", "K."),
                   family = "Ho",
                   role = c("aut", "ctb")))
Maintainer: Luis Garcia <luis.garcia@unb.br>
URL: https://github.com/lpfgarcia/ECoL/
Imports: cluster, e1071, FNN, igraph, MASS
Suggests: testthat
License: MIT + file LICENSE
BugReports: https://github.com/lpfgarcia/ECoL/issues
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 17:51:46 UTC; lpfgarcia
Author: Luis Garcia [aut, cre],
  Ana Lorena [aut, ctb],
  Marcilio Souto [aut, ctb],
  Tin K. Ho [aut, ctb]
Repository: CRAN
Date/Publication: 2026-05-20 06:50:02 UTC
Built: R 4.5.2; ; 2026-05-20 07:40:31 UTC; unix

Package: ECoL
Type: Package
Version: 0.4.4
Date: 2026-05-19
Title: Complexity Measures for Supervised Problems
Description: Provides measures to characterize the complexity of classification 
    and regression problems based on aspects that quantify the linearity of the 
    data, the presence of informative feature, the sparsity and dimensionality 
    of the datasets. This package provides bug fixes, generalizations and 
    implementations of many state of the art measures. The measures are 
    described in the papers: Lorena et al. (2019) <doi:10.1145/3347711> and 
    Lorena et al. (2018) <doi:10.1007/s10994-017-5681-1>.
Authors@R: c(person(given = "Luis",
                   family = "Garcia",
                   role = c("aut", "cre"),
                   email = "luis.garcia@unb.br"),
            person(given = "Ana",
                   family = "Lorena",
                   role = c("aut", "ctb")),
            person(given = "Marcilio",
                   family = "Souto",
                   role = c("aut", "ctb")),
            person(given = c("Tin", "K."),
                   family = "Ho",
                   role = c("aut", "ctb")))
Maintainer: Luis Garcia <luis.garcia@unb.br>
URL: https://github.com/lpfgarcia/ECoL/
Imports: cluster, e1071, FNN, igraph, MASS
Suggests: testthat
License: MIT + file LICENSE
BugReports: https://github.com/lpfgarcia/ECoL/issues
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 17:51:46 UTC; lpfgarcia
Author: Luis Garcia [aut, cre],
  Ana Lorena [aut, ctb],
  Marcilio Souto [aut, ctb],
  Tin K. Ho [aut, ctb]
Repository: CRAN
Date/Publication: 2026-05-20 06:50:02 UTC
Built: R 4.5.2; ; 2026-05-20 21:42:22 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(balance,default)
S3method(balance,formula)
S3method(complexity,default)
S3method(complexity,formula)
S3method(correlation,default)
S3method(correlation,formula)
S3method(dimensionality,default)
S3method(dimensionality,formula)
S3method(featurebased,default)
S3method(featurebased,formula)
S3method(linearity,default)
S3method(linearity,formula)
S3method(neighborhood,default)
S3method(neighborhood,formula)
S3method(network,default)
S3method(network,formula)
S3method(smoothness,default)
S3method(smoothness,formula)
export(balance)
export(complexity)
export(correlation)
export(dimensionality)
export(featurebased)
export(linearity)
export(neighborhood)
export(network)
export(smoothness)
export(summarization)

#' Measures of smoothness
#'
#' Regression task. In regression problems, the smoother the function to be 
#' fitted to the data, the simpler it shall be. Larger variations in the inputs
#' and/or outputs, on the other hand, usually indicate the existence of more 
#' intricate relationships between them.
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A response vector with one value for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the output column.
#' @param data A data.frame dataset contained the input and output attributes.
#' @param summary A list of summarization functions or empty for all values. See
#'  \link{summarization} method to more information. (Default: 
#'  \code{c("mean", "sd")})
#' @param ... Not used.
#' @details
#'  The following measures are allowed for this method:
#'  \describe{
#'    \item{"S1"}{Output distribution (S1) monitors whether the examples 
#'      joined in the MST have similar output values. Lower values indicate 
#'      simpler problems, where the outputs of similar examples in the input 
#'      space are also next to each other.}
#'    \item{"S2"}{Input distribution (S2) measure how similar in the input space
#'      are data items with similar outputs based on distance.}
#'    \item{"S3"}{Error of a nearest neighbor regressor (S3) calculates the mean
#'      squared error of a 1-nearest neighbor regressor  using leave-one-out.}
#'    \item{"S4"}{Non-linearity of nearest neighbor regressor (S4) calculates 
#'      the mean squared error of a 1-nearest neighbor regressor to the new 
#'      randomly interpolated points.}
#'  }
#' @return A list named by the requested smoothness measure.
#'
#' @references
#'  Ana C Lorena and Aron I Maciel and Pericles B C Miranda and Ivan G Costa and
#'    Ricardo B C Prudencio. (2018). Data complexity meta-features for 
#'    regression problems. Machine Learning, 107, 1, 209--246.
#'
#' @examples
#' ## Extract all smoothness measures for regression task
#' data(cars)
#' smoothness(speed ~ ., cars)
#' @export
smoothness <- function(...) {
  UseMethod("smoothness")
}

#' @rdname smoothness
#' @export
smoothness.default <- function(x, y, measures="all", summary=c("mean", "sd"), 
                               ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  if(is.factor(y)) {
    stop("label attribute needs to be numeric")
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.smoothness()
  }

  measures <- match.arg(measures, ls.smoothness(), TRUE)

  if (length(summary) == 0) {
    summary <- "return"
  }

  colnames(x) <- make.names(colnames(x), unique=TRUE)
  x <- normalize(x)
  y <- normalize(y)[,1]

  x <- x[order(y), , drop=FALSE]
  y <- y[order(y)]
  d <- dist(x)

  sapply(measures, function(f) {
    measure = eval(call(paste("r", f, sep="."), d=d, x=x, y=y))
    summarization(measure, summary, f %in% ls.smoothness.multiples(), ...)
  }, simplify=FALSE)
}

#' @rdname smoothness
#' @export
smoothness.formula <- function(formula, data, measures="all", 
                               summary=c("mean", "sd"), ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  smoothness.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    measures, summary, ...)
}

ls.smoothness <- function() {
  c("S1", "S2", "S3", "S4")
}

ls.smoothness.multiples <- function() {
  ls.smoothness()
}

r.S1 <- function(d, x, y) {

  g <- igraph::graph.adjacency(d, mode="undirected", weighted=TRUE)
  tree <- as.matrix(igraph::as_adj(igraph::mst(g)))
  tmp <- which(tree != 0, arr.ind=TRUE)
  aux <- abs(y[tmp[,1]] - y[tmp[,2]])
  aux/(aux + 1)
}

r.S2 <- function(d, x, y) {

  pred <- sapply(2:nrow(d), function(i) {
    d[i-1, i]
  })

  pred/(pred + 1)
}

r.S3 <- function(d, x, y) {

  diag(d) <- Inf
  pred <- apply(d, 1, function(i) {
    y[which.min(i)]
  })

  aux <- (pred - y)^2
  aux/(aux + 1)
}

r.S4 <- function(d, x, y) {
  test <- regr.generate(x, y, nrow(x))
  pred <- FNN::knn.reg(x, test[, -ncol(test), drop=FALSE], y, k=1)$pred
  aux <- (pred - test[, ncol(test)])^2
  aux/(aux + 1)
}

context("Test Smoothness")

test_that("regression.result", {

  aux = smoothness(speed ~ ., cars, measures="S1", summary="mean")
  expect_equal(as.numeric(aux$S1), 0.14745010, tolerance=0.1)

  aux = smoothness(speed ~ ., cars, measures="S2", summary="mean")
  expect_equal(as.numeric(aux$S2), 0.09936067)

  aux = smoothness(speed ~ ., cars, measures="S3", summary="mean")
  expect_equal(as.numeric(aux$S3), 0.03390902)

  set.seed(123)
  aux = smoothness(speed ~ ., cars, measures="S4", summary="mean")
  expect_equal(as.numeric(aux$S4), 0.03171115)
})

test_that("validation.error",{

  expect_error(smoothness(speed ~ ., cars, measures="C2"))
  expect_error(smoothness(speed ~ ., cars, measures="C2", summary="abc"))

  expect_error(smoothness(Species ~ ., iris))
  expect_error(smoothness(Species ~ ., iris, measures="C2"))
  expect_error(smoothness(Species ~ ., iris, measures="C2", summary="abc"))
})

#' Measures of feature-based
#'
#' Classification task. The feature-based measures evaluate how informative the 
#' available features are to separate the classes. If there is at least one very
#' discriminative feature in the dataset, the problem can be considered simpler 
#' than if there is no such an attribute. 
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A factor response vector with one label for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the class column.
#' @param data A data.frame dataset contained the input attributes and class.
#' @param summary A list of summarization functions or empty for all values. See
#'  \link{summarization} method to more information. (Default: 
#'  \code{c("mean", "sd")})
#' @param ... Not used.
#' @details
#'  The following measures are allowed for this method:
#'  \describe{
#'    \item{"F1"}{Maximum Fisher's Discriminant Ratio (F1) measures the overlap 
#'      between the values of the features and takes the value of the largest 
#'      discriminant ratio among all the available features.}
#'    \item{"F1v"}{Directional-vector maximum Fisher's discriminant ratio (F1v)
#'      complements F1 by searching for a vector able to separate two classes 
#'      after the training examples have been projected into it.}
#'    \item{"F2"}{Volume of the overlapping region (F2) computes the overlap of 
#'      the distributions of the features values within the classes. F2 can be 
#'      determined by finding, for each feature its minimum and maximum values 
#'      in the classes.}
#'    \item{"F3"}{The maximum individual feature efficiency (F3) of each 
#'      feature is given by the ratio between the number of examples that are 
#'      not in the overlapping region of two classes and the total number of 
#'      examples. This measure returns the maximum of the values found among 
#'      the input features.}
#'    \item{"F4"}{Collective feature efficiency (F4) get an overview on how 
#'      various features may work together in data separation. First the most 
#'      discriminative feature according to F3 is selected and all examples that
#'      can be separated by this feature are removed from the dataset. The 
#'      previous step is repeated on the remaining dataset until all the 
#'      features have been considered or no example remains. F4 returns the 
#'      ratio of examples that have been discriminated.}
#'  }
#' @return A list named by the requested feature-based measure.
#'
#' @references
#'  Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
#'    data complexity library in C++. Technical Report. La Salle - Universitat
#'    Ramon Llull.
#'
#' @examples
#' ## Extract all feature-based measures for classification task
#' data(iris)
#' featurebased(Species ~ ., iris)
#' @export
featurebased <- function(...) {
  UseMethod("featurebased")
}

#' @rdname featurebased
#' @export
featurebased.default <- function(x, y, measures="all", summary=c("mean", "sd"), 
                                ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  y <- as.factor(y)

  if(min(table(y)) < 2) {
    stop("number of examples in the minority class should be >= 2")
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.featurebased()
  }

  measures <- match.arg(measures, ls.featurebased(), TRUE)

  if (length(summary) == 0) {
    summary <- "return"
  }

  colnames(x) <- make.names(colnames(x), unique=TRUE)
  x <- binarize(x)
  data <- data.frame(x, class=y)

  sapply(measures, function(f) {
    measure = eval(call(paste("class", f, sep="."), data=data))
    summarization(measure, summary, f %in% ls.featurebased.multiples(), ...)
  }, simplify=FALSE)
}

#' @rdname featurebased
#' @export
featurebased.formula <- function(formula, data, measures="all", 
                                summary=c("mean", "sd"), ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  featurebased.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    measures, summary, ...)
}

ls.featurebased <- function() {
  c("F1", "F1v", "F2", "F3", "F4")
}

ls.featurebased.multiples <- function() {
  ls.featurebased()
}

branch <- function(data, j) {
  data[data$class == j, -ncol(data), drop=FALSE]
}

numerator <- function(j, data) {

  tmp <- branch(data, j)
  aux <- nrow(tmp) * (colMeans(tmp) - 
    colMeans(data[,-ncol(data), drop=FALSE]))^2
  return(aux)
}

denominator <- function(j, data) {

  tmp <- branch(data, j)
  aux <- rowSums((t(tmp) - colMeans(tmp))^2)
  return(aux)
}

class.F1 <- function(data) {

  num <- lapply(levels(data$class), numerator, data)
  den <- lapply(levels(data$class), denominator, data)

  aux <- rowSums(do.call("cbind", num)) / 
    rowSums(do.call("cbind", den))

  #aux <- max(aux, na.rm=TRUE)
  aux <- 1/(aux + 1)
  return(aux)
}

dvector <- function(data) {

  l <- levels(data$class)
  a <- branch(data, l[1])
  b <- branch(data, l[2])

  c1 <- colMeans(a)
  c2 <- colMeans(b)

  W <- (nrow(a)/nrow(data)) * stats::cov(a) + 
    (nrow(b)/nrow(data)) * stats::cov(b)

  B <- (c1 - c2) %*% t(c1 - c2)
  d <- MASS::ginv(W) %*% (c1 - c2)

  aux <- (t(d) %*% B %*% d)/(t(d) %*% W %*% d)
  return(aux)
}

class.F1v <- function(data) {
  data <- ovo(data)
  #aux <- mean(sapply(data, dvector))
  aux <- sapply(data, dvector)
  aux <- 1/(aux + 1)
  return(aux)
}

regionOver <- function(data) {

  l <- levels(data$class)
  a <- branch(data, l[1])
  b <- branch(data, l[2])

  maxmax <- rbind(colMax(a), colMax(b))
  minmin <- rbind(colMin(a), colMin(b))

  over <- colMax(rbind(colMin(maxmax) - colMax(minmin), 0))
  rang <- colMax(maxmax) - colMin(minmin)
  aux <- prod(over/rang, na.rm=TRUE)
  return(aux)
}

class.F2 <- function(data) {

  data <- ovo(data)
  #aux <- mean(sapply(data, regionOver))
  aux <- sapply(data, regionOver)
  return(aux)
}

nonOverlap <- function(data) {

  l <- levels(data$class)
  a <- branch(data, l[1])
  b <- branch(data, l[2])

  minmax <- colMin(rbind(colMax(a), colMax(b)))
  maxmin <- colMax(rbind(colMin(a), colMin(b)))

  aux <- do.call("cbind",
    lapply(1:(ncol(data)-1), function(i) {
        data[,i] < maxmin[i] | data[,i] > minmax[i]
    })
  )

  aux <- data.frame(aux)
  rownames(aux) <- rownames(data)
  return(aux)
}

class.F3 <- function(data) {

  data <- ovo(data)
  aux <- mapply(function(d) {
    colSums(nonOverlap(d))/nrow(d)
  }, d=data)

  #aux <- 1 - mean(colMax(aux))
  aux <- 1 - colMax(aux)
  return(aux)
}

removing <- function(data) {

  repeat {

    tmp <- nonOverlap(data)
    col <- which.max(colSums(tmp))
    aux <- rownames(tmp[tmp[,col] != TRUE, , drop=FALSE])
    data <- data[aux,- col, drop=FALSE]

    if(nrow(data) == 0 | ncol(data) == 1 |
      length(unique(data$class)) == 1)
        break
  }

  return(data)
}

class.F4 <- function(data) {

  data <- ovo(data)
  aux <- mapply(function(d) {
    nrow(removing(d))/nrow(d)
  }, d=data)

  #aux <- mean(aux)
  return(aux)
}

context("Test Overlapping")

test_that("multiclass.result", {

  aux = featurebased(Species ~ ., iris, measures="F1", summary="return")
  aux = 1/(max((1/aux$F1) - 1) + 1)
  expect_equal(aux, 0.05862828)

  aux = featurebased(Species ~ ., iris, measures="F1v", summary="return")
  aux = 1/(mean((1/aux$F1v) - 1) + 1)
  expect_equal(aux, 0.009504561)

  aux = featurebased(Species ~ ., iris, measures="F2", summary="mean")
  expect_equal(as.numeric(aux$F2), 0.006381766)

  aux = featurebased(Species ~ ., iris, measures="F3", summary="mean")
  expect_equal(as.numeric(aux$F3), 0.123333333)

  aux = featurebased(Species ~ ., iris, measures="F4", summary="mean")
  expect_equal(as.numeric(aux$F4), 0.043333333)
})

test_that("binary.result", {

  iris = iris[1:100,]
  iris$Species = factor(iris$Species)

  aux = featurebased(Species ~ ., iris, measures="F1", summary="return")
  aux = 1/(max((1/aux$F1) - 1) + 1)
  expect_equal(aux, 0.059118951)

  aux = featurebased(Species ~ ., iris, measures="F1v", summary="return")
  aux = 1/(mean((1/aux$F1v) - 1) + 1)
  expect_equal(aux, 0.009593841)

  aux = featurebased(Species ~ ., iris, measures="F2", summary="mean")
  expect_equal(as.numeric(aux$F2), 0)

  aux = featurebased(Species ~ ., iris, measures="F3", summary="mean")
  expect_equal(as.numeric(aux$F3), 0)

  aux = featurebased(Species ~ ., iris, measures="F4", summary="mean")
  expect_equal(as.numeric(aux$F4), 0)
})

test_that("validation.error",{

  expect_error(featurebased(Species ~ ., iris, measures="L1"))
  expect_error(featurebased(Species ~ ., iris, measures="L2", summary="abc"))

  expect_error(featurebased(speed ~ ., cars))
  expect_error(featurebased(speed ~ ., cars, measures="L1"))
  expect_error(featurebased(speed ~ ., cars, measures="L2", summary="abc"))
})

#' Extract the complexity measures from datasets
#'
#' This function is responsable to extract the complexity measures from the 
#' classification and regression tasks. For such, they take into account the 
#' overlap between classes imposed by feature values, the separability and 
#' distribution of the data points and the value of structural measures based on
#' the representation of the dataset as a graph structure. To set specific 
#' parameters for each group, use the characterization function.
#'
#' @param x A data.frame contained only the input attributes.
#' @param y A response vector with one value for each row/component of x.
#' @param groups A list of complexity measures groups or \code{"all"} to include
#'  all of them.
#' @param formula A formula to define the output column.
#' @param data A data.frame dataset contained the input and output attributes.
#' @param summary A list of summarization functions or empty for all values. See
#'  \link{summarization} method to more information. (Default: 
#'  \code{c("mean", "sd")})
#' @param ... Not used.
#' @details
#'  The following groups are allowed for this method:
#'  \describe{
#'    \item{"featurebased"}{The feature feature-based measures characterize how 
#'      informative the available features are to separate the classes See 
#'      \link{featurebased} for more details.}
#'    \item{"neighborhood"}{Neighborhood measures characterize the presence and 
#'      density of same or different classes in local neighborhoods. See 
#'      \link{neighborhood} for more details.}
#'    \item{"linearity"}{Linearity measures try to quantify whether the labels 
#'      can be linearly separated. See \link{linearity} for more details.}
#'    \item{"dimensionality"}{The dimensionality measures compute information on
#'      how smoothly the examples are distributed within the attributes. See 
#'      \link{dimensionality} for more details.}
#'    \item{"balance"}{Class balance measures take into account the numbers of 
#'      examples per class in the dataset. See \link{balance} for more details.}
#'    \item{"network"}{Network measures represent the dataset as a graph and 
#'      extract structural information from it. See \link{network} for more 
#'      details.}
#'    \item{"correlation"}{Capture the relationship of the feature values with 
#'      the outputs. See \link{correlation} for more details.}
#'    \item{"smoothness"}{Estimate the smoothness of the function that must be 
#'      fitted to the data. See \link{smoothness} for more details.}
#'  }
#' @return A numeric vector named by the requested complexity measures.
#'
#' @references
#'  Tin K Ho and Mitra Basu. (2002). Complexity measures of supervised 
#'    classification problems. IEEE Transactions on Pattern Analysis and Machine
#'    Intelligence, 24, 3, 289--300.
#'
#'  Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for 
#'    the data complexity library in C++. Technical Report. La Salle - 
#'    Universitat Ramon Llull.
#'
#'  Ana C Lorena and Aron I Maciel and Pericles B C Miranda and Ivan G Costa and
#'    Ricardo B C Prudencio. (2018). Data complexity meta-features for 
#'    regression problems. Machine Learning, 107, 1, 209--246.
#'
#' @examples
#' ## Extract all complexity measures for classification task
#' data(iris)
#' complexity(Species ~ ., iris)
#'
#' ## Extract all complexity measures for regression task
#' data(cars)
#' complexity(speed ~ ., cars)
#' @export
complexity <- function(...) {
  UseMethod("complexity")
}

#' @rdname complexity
#' @export
complexity.default <- function(x, y, groups="all", summary=c("mean", "sd"), 
                               ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  type <- "regr"
  if(is.factor(y)) {
    type <- "class"
    if(min(table(y)) < 2) {
      stop("number of examples in the minority class should be >= 2")
    }
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(groups[1] == "all") {
    groups <- ls.complexity(type)
  }

  groups <- match.arg(groups, ls.complexity(type), TRUE)

  if (length(summary) == 0) {
    summary <- "return"
  }

  colnames(x) <- make.names(colnames(x), unique=TRUE)

  unlist(
    sapply(groups, function(group) {
      do.call(group, list(x=x, y=y, summary=summary, ...))
    }, simplify=FALSE)
  )
}

#' @rdname complexity
#' @export
complexity.formula <- function(formula, data, groups="all", 
                               summary=c("mean", "sd"), ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  complexity.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    groups, summary, ...)
}

ls.complexity <- function(type) {

  switch(type,
    class = {
      c("featurebased", "neighborhood", 
        "linearity", "dimensionality",
        "balance", "network")
    }, regr = {
      c("correlation", "linearity", 
        "smoothness", "dimensionality")
    }
  )
}

context("Test Complexity")

test_that("multiclass.result", {

  set.seed(123)
  aux = complexity(Species ~ ., iris)

  set.seed(123)
  expect_equal(aux, complexity(iris[1:4], iris[5]))
})

test_that("binary.result", {

  iris = iris[1:100,]
  iris$Species = factor(iris$Species)

  set.seed(123)
  aux = complexity(Species ~ ., iris)

  set.seed(123)
  expect_equal(aux, complexity(iris[1:4], iris[5]))
})

test_that("regression.result", {

  set.seed(123)
  aux = complexity(speed ~ ., cars)

  set.seed(123)
  expect_equal(aux, complexity(cars[2], cars[1]))
})

test_that("validation.error",{

  expect_error(complexity(iris[1:130, 1:4], iris[5]))
  expect_error(complexity(iris[,1:4], factor(c(rep(1, 149), rep(2, 1)))))
  expect_error(complexity(as.matrix(iris[, c(1,2)]), iris$Species))
  expect_error(complexity(Species ~ ., iris, groups="abc"))
  expect_error(complexity(Species ~ ., runif(100)))
})

#' Measures of dimensionality
#'
#' These measures give an indicative of data sparsity. They capture how sparse 
#' a datasets tend to have regions of low density. These regions are know to be 
#' more difficult to extract good classification and regression models.
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A response vector with one value for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the output column.
#' @param data A data.frame dataset contained the input and output attributes.
#' @param ... Not used.
#' @details
#'  The following measures are allowed for this method:
#'  \describe{
#'    \item{"D1"}{Average number of points per dimension (D1) is given by the 
#'      ratio between the number of examples and dimensionality of the dataset.}
#'    \item{"D2"}{Average number of points per PCA (D2) is similar to T2, but 
#'      uses the number of PCA components needed to represent 95% of data 
#'      variability as the base of data sparsity assessment.}
#'    \item{"D3"}{Ratio of the PCA Dimension to the Original (D3) estimates the
#'      proportion of relevant and the original dimensions for a dataset.}
#'  }
#' @return A list named by the requested dimensionality measure.
#'
#' @references
#'  Ana C Lorena, Ivan G Costa, Newton Spolaor and Marcilio C P Souto. (2012). 
#'    Analysis of complexity indices for classification problems: Cancer gene 
#'    expression data. Neurocomputing 75, 1, 33--42.
#'
#' @examples
#' ## Extract all dimensionality measures for classification task
#' data(iris)
#' dimensionality(Species ~ ., iris)
#'
#' ## Extract all dimensionality measures for regression task
#' data(cars)
#' dimensionality(speed ~ ., cars)
#' @export
dimensionality <- function(...) {
  UseMethod("dimensionality")
}

#' @rdname dimensionality
#' @export
dimensionality.default <- function(x, y, measures="all", ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.dimensionality()
  }

  measures <- match.arg(measures, ls.dimensionality(), TRUE)
  colnames(x) <- make.names(colnames(x), unique=TRUE)

  x <- binarize(x)

  sapply(measures, function(f) {
    eval(call(paste("class", f, sep="."), x=x))
  })
}

#' @rdname dimensionality
#' @export
dimensionality.formula <- function(formula, data, measures="all", ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  dimensionality.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    measures, ...)
}

ls.dimensionality <- function() {
  c("D1", "D2", "D3")
}

pca <- function(x) {
  aux <- stats::prcomp(x)
  tmp <- length(which(summary(aux)$importance[3,] < 0.95)) + 1
  return(tmp)
}

class.D1 <- function(x) {
  ncol(x)/nrow(x)
}

class.D2 <- function(x) {
  pca(x)/nrow(x)
}

class.D3 <- function(x) {
  pca(x)/ncol(x)
}

context("Test Dimensionality")

test_that("data.result", {

  aux = dimensionality(Species ~ ., iris)
  expect_equal(as.numeric(aux), c(0.02666667, 0.01333333, 0.50000000))

  aux = dimensionality(speed ~ ., cars)
  expect_equal(as.numeric(aux), c(0.02, 0.02, 1))
})


test_that("validation.error",{

  expect_error(dimensionality(Species ~ ., iris, measures="N1"))
  expect_error(dimensionality(Species ~ ., iris, measures="N1", summary="abc"))
})

#' Measures of neighborhood
#'
#' Classification task. The Neighborhood measures analyze the neighborhoods of 
#' the data items and try to capture class overlapping and the shape of the 
#' decision boundary. They work over a distance matrix storing the distances 
#' between all pairs of data points in the dataset.
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A factor response vector with one label for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the class column.
#' @param data A data.frame dataset contained the input attributes and class.
#' @param summary A list of summarization functions or empty for all values. See
#'  \link{summarization} method to more information. (Default: 
#'  \code{c("mean", "sd")})
#' @param ... Not used.
#' @details
#'  The following measures are allowed for this method:
#'  \describe{
#'    \item{"N1"}{Fraction of borderline points (N1) computes the percentage of 
#'      vertexes incident to edges connecting examples of opposite classes in 
#'      a Minimum Spanning Tree (MST).}
#'    \item{"N2"}{Ratio of intra/extra class nearest neighbor distance (N2)  
#'      computes the ratio of two sums: intra-class and inter-class. The former 
#'      corresponds to the sum of the distances between each example and its 
#'      closest neighbor from the same class. The later is the sum of the 
#'      distances between each example and its closest neighbor from another 
#'      class (nearest enemy).}
#'    \item{"N3"}{Error rate of the nearest neighbor (N3) classifier corresponds
#'      to the error rate of a one Nearest Neighbor (1NN) classifier, estimated 
#'      using a leave-one-out procedure in dataset.}
#'    \item{"N4"}{Non-linearity of the nearest neighbor classifier (N4) creates 
#'      a new dataset randomly interpolating pairs of training examples of the 
#'      same class and then induce a the 1NN classifier on the original data and
#'      measure the error rate in the new data points.}
#'    \item{"N5"}{Fraction of hyperspheres covering data (N5) builds 
#'      hyperspheres centered at each one of the training examples, which have 
#'      their radios growth until the hypersphere reaches an example of another 
#'      class. Afterwards, smaller hyperspheres contained in larger hyperspheres 
#'      are eliminated. N5 is finally defined as the ratio between the number of 
#'      the remaining hyperspheres and the total number of examples in the 
#'      dataset.}
#'    \item{"N6"}{Local Set Average Cardinality (N6) is based on Local Set 
#'      (LS) and defined as the set of points from the dataset whose distance of
#'      each example is smaller than the distance from the exemples of the 
#'      different class. LSC is the average of the LS.}
#'  }
#' @return A list named by the requested neighborhood measure.
#'
#' @references
#'  Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
#'    data complexity library in C++. Technical Report. La Salle - Universitat 
#'    Ramon Llull.
#'
#'  Enrique Leyva, Antonio Gonzalez and Raul Perez. (2014). A Set of Complexity
#'    Measures Designed for Applying Meta-Learning to Instance Selection. IEEE
#'    Transactions on Knowledge and Data Engineering 27, 2, 354--367.
#'
#' @examples
#' ## Extract all neighborhood measures for classification task
#' data(iris)
#' neighborhood(Species ~ ., iris)
#' @export
neighborhood <- function(...) {
  UseMethod("neighborhood")
}

#' @rdname neighborhood
#' @export
neighborhood.default <- function(x, y, measures="all", summary=c("mean", "sd"), 
                                 ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  y <- as.factor(y)

  if(min(table(y)) < 2) {
    stop("number of examples in the minority class should be >= 2")
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.neighborhood()
  }

  measures <- match.arg(measures, ls.neighborhood(), TRUE)

  if (length(summary) == 0) {
    summary <- "return"
  }

  colnames(x) <- make.names(colnames(x), unique=TRUE)
  data <- data.frame(x, class=y)
  dst <- dist(x)

  sapply(measures, function(f) {
    measure = eval(call(paste("class", f, sep="."), dst=dst, data=data))
    summarization(measure, summary, f %in% ls.neighborhood.multiples(), ...)
  }, simplify=FALSE)
}

#' @rdname neighborhood
#' @export
neighborhood.formula <- function(formula, data, measures="all", 
                                 summary=c("mean", "sd"), ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  neighborhood.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    measures, summary, ...)
}

ls.neighborhood <- function() {
  c("N1","N2", "N3", "N4", "N5", "N6")
}

ls.neighborhood.multiples <- function() {
  c("N2", "N3", "N4")
}

class.N1 <- function(dst, data) {

  g <- igraph::graph.adjacency(dst, mode="undirected", weighted=TRUE)
  tree <- as.matrix(igraph::as_adj(igraph::mst(g)))

  tmp <- which(tree != 0, arr.ind=TRUE)
  aux <- which(data[tmp[,1],]$class != data[tmp[,2],]$class)
  aux <- length(unique(tmp[aux,1]))
  return(aux/nrow(data))
}

intra <- function(dst, data, i) {
  tmp <- rownames(data[data$class == data[i,]$class,])
  aux <- min(dst[i, setdiff(tmp, i)])
  return(aux)
}

inter <- function(dst, data, i) {
  tmp <- rownames(data[data$class != data[i,]$class,])
  aux <- sort(dst[i, tmp])[1]
  return(aux)
}

class.N2 <- function(dst, data) {

  aux <- sapply(rownames(data), function(i) {
    c(intra(dst, data, i), inter(dst, data, i))
  })

  #aux <- sum(aux[1,])/sum(aux[2,])
  aux <- 1 - (1/((aux[1,]/aux[2,]) + 1))
  return(aux)
}

knn <- function(data, dst, k) {
  apply(dst, 1, function(i) {
    tmp <- names(sort(i)[k])
    data[tmp,]$class
  })
}

class.N3 <- function(dst, data) {
  aux <- knn(data, dst, 2) != data$class
  return(mean(aux))
}

class.N4 <- function(dst, data) {

  tran <- rbind(data, class.generate(data, nrow(data)))
  test <- utils::tail(tran, nrow(data))

  dst <- dist(tran[,-ncol(tran), drop=FALSE])
  dst <- dst[rownames(test), rownames(data)]

  aux <- knn(data, dst, 1) != test$class
  return(mean(aux))
}

radios <- function(dst, data, i) {

  di <- inter(dst, data, i)
  j <- names(di)
  dj <- inter(dst, data, j)
  k <- names(dj)

  if(i == k) {
    return(di/2)
  } else {
    tmp <- radios(dst, data, j)
    return(di - tmp)
  }
}

hyperspher <- function(dst, data) {

  aux <- sapply(rownames(data), function(i) {
    as.numeric(radios(dst, data, i))
  })

  return(aux)
}

translate <- function(dst, r) {

  aux <- t(sapply(rownames(dst), function(i) {
    dst[i,] < r[i]
  }))

  return(aux)
}

adherence <- function(adh, data) {

  h <- n <- c()

  repeat{

    aux <- which.max(rowSums(adh))
    tmp <- names(which(adh[aux,]))
    dif <- setdiff(rownames(adh), c(tmp, names(aux)))
    adh <- adh[dif, dif, drop=FALSE]

    if(all(dim(adh) != 0)) {
      h <- c(h, length(tmp))
    } else {
      h <- c(h, 1)
    }

    n <- c(n, names(aux))

    if(all(dim(adh)) == 0)
      break
  }

  names(h) <- n
  return(h)
}

class.N5 <- function(dst, data) {
  r <- hyperspher(dst, data)
  aux <- adherence(translate(dst, r), data)
  return(length(aux)/nrow(data))
}

class.N6 <- function(dst, data) {

  r <- sapply(rownames(data), function(i) {
    as.numeric(inter(dst, data, i))
  })

  aux <- 1 - sum(translate(dst, r))/(nrow(dst)^2)
  return(aux)
}

context("Test Neighborhood")

test_that("multiclass.result", {

  aux = neighborhood(Species ~ ., iris, measures="N1", summary="mean")
  expect_equal(as.numeric(aux$N1), 0.106666667)

  aux = neighborhood(Species ~ ., iris, measures="N2", summary="mean")
  expect_equal(as.numeric(aux$N2), 0.1981444, tolerance=0.001)

  aux = neighborhood(Species ~ ., iris, measures="N3", summary="mean")
  expect_equal(as.numeric(aux$N3), 0.060000000)

  set.seed(123)
  aux = neighborhood(Species ~ ., iris, measures="N4", summary="mean")
  expect_equal(as.numeric(aux$N4), 0.00666667)

  aux = neighborhood(Species ~ ., iris, measures="N5", summary="return")
  expect_equal(aux$N5, 0.12)

  aux = neighborhood(Species ~ ., iris, measures="N6", summary="mean")
  expect_equal(as.numeric(aux$N6), 0.816400000)
})

test_that("validation.error",{

  expect_error(neighborhood(Species ~ ., iris, measures="C1"))
  expect_error(neighborhood(Species ~ ., iris, measures="C1", summary="abc"))

  expect_error(neighborhood(speed ~ ., cars))
  expect_error(neighborhood(speed ~ ., cars, measures="C1"))
  expect_error(neighborhood(speed ~ ., cars, measures="C1", summary="abc"))
})

#' Measures of class balance
#'
#' Classification task. These measures capture the differences in the number of 
#' examples per class in the dataset. When these differences are severe, 
#' problems related to generalization of the ML classification techniques could 
#' happen because of the imbalance ratio.
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A factor response vector with one label for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the class column.
#' @param data A data.frame dataset contained the input attributes and class.
#' @param ... Not used.
#' @details
#'  The following measures are allowed for this method:
#'  \describe{
#'    \item{"B1"}{The entropy of class proportions (B1) capture the imbalance in
#'      a dataset based on the proportions of examples per class.}
#'    \item{"B2"}{The imbalance ratio (B2) is an index computed for measuring
#'      class balance. This is a version of the measure that is also suited for 
#'      multiclass classification problems.}
#'  }
#' @return A list named by the requested class balance measure.
#'
#' @references
#'  Ana C Lorena, Ivan G Costa, Newton Spolaor and Marcilio C P Souto. (2012). 
#'    Analysis of complexity indices for classification problems: Cancer gene 
#'    expression data. Neurocomputing 75, 1, 33--42.
#'
#'  Ajay K Tanwani and Muddassar Farooq. (2010). Classification potential vs. 
#'    classification accuracy: a comprehensive study of evolutionary algorithms 
#'    with biomedical datasets. Learning Classifier Systems 6471, 127--144.
#'
#' @examples
#' ## Extract all balance measures for classification task
#' data(iris)
#' balance(Species ~ ., iris)
#' @export
balance <- function(...) {
  UseMethod("balance")
}

#' @rdname balance
#' @export
balance.default <- function(x, y, measures="all", ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  y <- as.factor(y)

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.balance()
  }

  measures <- match.arg(measures, ls.balance(), TRUE)

  sapply(measures, function(f) {
    eval(call(paste("class", f, sep="."), y=y))
  },  simplify=FALSE)
}

#' @rdname balance
#' @export
balance.formula <- function(formula, data, measures="all", ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  balance.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    measures, ...)
}

ls.balance <- function() {
  c("B1", "B2")
}

class.B1 <- function(y) {
  c <- -1/log(nlevels(y))
  i <- table(y)/length(y)
  aux <- 1 - c*sum(i*log(i))
  return(aux)
}

class.B2 <- function(y) {
  ii <- summary(y)
  nc <- length(ii)
  aux <- ((nc - 1)/nc) * sum(ii/(length(y) - ii))
  aux <- 1 - (1/aux)
  return(aux)
}

context("Test Balance")

test_that("multiclass1.result", {

  aux = balance(Species ~ ., iris, measures="B1")
  expect_equal(as.numeric(aux$B1), 2.220446e-16)

  aux = balance(Species ~ ., iris, measures="B2")
  expect_equal(as.numeric(aux$B2), 0)
})

test_that("multiclass2.result", {

  class = c(rep(c(1,2), each=40), rep(3, 20)) 
  data = data.frame(x=runif(100), class=factor(class))

  aux = balance(class ~ ., data, measures="B1")
  expect_equal(as.numeric(aux$B1), 0.03977028)

  aux = balance(class ~ ., data, measures="B2")
  expect_equal(as.numeric(aux$B2), 0.05263158)
})

test_that("binary.result", {

  iris = iris[1:100,]
  iris$Species = factor(iris$Species)

  aux = balance(Species ~ ., iris, measures="B1")
  expect_equal(as.numeric(aux$B1), 0)

  aux = balance(Species ~ ., iris, measures="B2")
  expect_equal(as.numeric(aux$B2), 0)
})

test_that("validation.error",{

  expect_error(balance(Species ~ ., iris, measures="L1"))
  expect_error(balance(Species ~ ., iris, measures="L2", summary="abc"))

  expect_error(overlapping(speed ~ ., cars))
  expect_error(balance(speed ~ ., cars, measures="L1"))
  expect_error(balance(speed ~ ., cars, measures="L2", summary="abc"))
})

colMin <- function(x) {
  apply(x, 2, min)
}

colMax <- function(x) {
  apply(x, 2, max)
}

dist <- function(x) {
  as.matrix(cluster::daisy(x, metric="gower", stand=TRUE, warnBin=FALSE))
}

form <- function(x) {
  att <- paste(colnames(x), collapse="+")
  stats::formula(paste("~ 0 +", att, sep=" "))
}

binarize <- function(x) {
  data.frame(stats::model.matrix(form(x), x))
}

ovo <- function(data) {

  aux <- utils::combn(levels(data$class), 2)

  tmp <- apply(aux, 2, function(i) {
    vet <- base::subset(data, data$class %in% i)
    vet$class <- factor(vet$class)
    return(vet)
  })

  return(tmp)
}

class.interpolation <- function(data) {

  aux <- data[data$class == sample(data$class, 1),]
  tmp <- aux[sample(nrow(aux), 2),]

  rnd <- stats::runif(1)

  for(i in 1:(ncol(data)-1)) {
    if(is.numeric(data[,i])) {
      tmp[1,i] <- tmp[1,i] + (tmp[2,i] - tmp[1,i]) * rnd
    } else {
      tmp[1,i] <- sample(aux[,i], 1)
    }
  }

  return(tmp[1,])
}

class.generate <- function(data, n) {

  tmp <- do.call("rbind",
    lapply(1:n, function(i) {
      class.interpolation(data)
    })
  )

  return(tmp)
}

maxmin <- function(x) {
  (x - min(x))/(max(x) - min(x))
}

normalize <- function(x) {

  x <- as.data.frame(x)
  for(i in 1:ncol(x)) {
    if(is.numeric(x[,i]))
      if(length(unique(x[,i])) != 1)
        x[,i] <- maxmin(x[,i])
  }
  return(x)
}

spearman <- function(x) {
  1-6*sum(x^2)/(length(x)^3 - length(x))
}

regr.interpolation <- function(x, y, i) {

  aux <- x[(i-1):i,,drop=FALSE]

  rnd <- stats::runif(1)
  for(j in 1:ncol(x)) {
    if(is.numeric(x[,j])) {
      aux[1,j] <- aux[1,j] + (aux[2,j] - aux[1,j]) * rnd
    } else {
      aux[1,j] <- sample(aux[,j], 1)
    }
  }

  tmp <- y[(i-1):i]
  rnd <- stats::runif(1)
  tmp[1] <- tmp[1]*rnd + tmp[2]*(1-rnd)

  return(cbind(aux[1,], tmp[1]))
}

regr.generate <- function(x, y, n) {

  tmp <- do.call("rbind",
    lapply(2:n, function(i) {
      regr.interpolation(x, y, i)
    })
  )

  tmp <- data.frame(tmp)
  colnames(tmp) <- c(colnames(x), "y")
  return(tmp)
}

#' Measures of network
#'
#' Classification task. The network measures represent the dataset as a graph 
#' and extract structural information from it. The transformation between raw 
#' data and the graph representation is based on the epsilon-NN algorithm. Next,
#' a post-processing step is applied to the graph, pruning edges between 
#' examples of opposite classes.
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A factor response vector with one label for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the class column.
#' @param data A data.frame dataset contained the input attributes and class.
#' @param eps The percentage of nodes in the graph to be connected.
#' @param summary A list of summarization functions or empty for all values. See
#'  \link{summarization} method to more information. (Default: 
#'  \code{c("mean", "sd")})
#' @param ... Not used.
#' @details
#'  The following measures are allowed for this method:
#'  \describe{
#'    \item{"G1"}{Average Density of the network (G1) represents the 
#'      number of edges in the graph, divided by the maximum number of edges 
#'      between pairs of data points.}
#'    \item{"G2"}{Clustering coefficient (G2) averages the clustering 
#'      tendency of the vertexes by the ratio of existent edges between its 
#'      neighbors and the total number of edges that could possibly exist 
#'      between them.}
#'    \item{"G3"}{Hubs score (G3) is given by the number of connections it  
#'      has to other nodes, weighted by the number of connections these 
#'      neighbors have.}
#'  }
#' @return A list named by the requested network measure.
#'
#' @references
#'  Gleison Morais and Ronaldo C Prati. (2013). Complex Network Measures for 
#'    Data Set Characterization. In 2nd Brazilian Conference on Intelligent 
#'    Systems (BRACIS). 12--18.
#'
#'  Luis P F Garcia, Andre C P L F de Carvalho and Ana C Lorena. (2015). Effect
#'    of label noise in the complexity of classification problems. 
#'    Neurocomputing 160, 108--119.
#'
#' @examples
#' ## Extract all network measures for classification task
#' data(iris)
#' network(Species ~ ., iris)
#' @export
network <- function(...) {
  UseMethod("network")
}

#' @rdname network
#' @export
network.default <- function(x, y, measures="all", eps=0.15, 
                            summary=c("mean", "sd"), ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  y <- as.factor(y)

  if(min(table(y)) < 2) {
    stop("number of examples in the minority class should be >= 2")
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.network()
  }

  measures <- match.arg(measures, ls.network(), TRUE)

  if (length(summary) == 0) {
    summary <- "return"
  }

  colnames(x) <- make.names(colnames(x), unique=TRUE)
  dst <- enn(x, y, eps*nrow(x))
  graph <- igraph::graph.adjacency(dst, mode="undirected", weighted=TRUE)

  sapply(measures, function(f) {
    measure = eval(call(paste("class", f, sep="."), graph))
    summarization(measure, summary, f %in% ls.network.multiples(), ...)
  }, simplify=FALSE)
}

#' @rdname network
#' @export
network.formula <- function(formula, data, measures="all", eps=0.15, 
                            summary=c("mean", "sd"), ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  network.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    measures, eps, summary, ...)
}

ls.network <- function() {
  c("G1", "G2", "G3")
}

ls.network.multiples <- function() {
  c("G3")
}

enn <- function(x, y, e) {

  dst <- dist(x)

  for(i in 1:nrow(x)) {
    a <- names(sort(dst[i,])[1:e+1])
    b <- rownames(x[y == y[i],])
    dst[i, setdiff(rownames(x), intersect(a, b))] <- 0
  }

  return(dst)
}

class.G1 <- function(graph) {
  1 - igraph::graph.density(graph)
}

class.G2 <- function(graph) {
  1 - igraph::transitivity(graph, type="global", isolates="zero")
}

class.G3 <- function(graph) {
  #1 - mean(igraph::hub.score(graph)$vector)
  1 - igraph::hub.score(graph)$vector
}

context("Test Network")

test_that("multiclass.result", {

  aux = network(Species ~ ., iris, measures="G1")
  expect_equal(as.numeric(aux$G1), 0.8332886)

  aux = network(Species ~ ., iris, measures="G2")
  expect_equal(as.numeric(aux$G2), 0.267977, tolerance=0.01)

  aux = network(Species ~ ., iris, measures="G3", summary="mean")
  expect_equal(as.numeric(aux$G3), 0.8380508, tolerance=0.01)
})

test_that("validation.error",{

  expect_error(network(Species ~ ., iris, measures="T1"))
  expect_error(network(Species ~ ., iris, measures="T1", summary="abc"))

  expect_error(network(speed ~ ., cars))
  expect_error(network(speed ~ ., cars, measures="T1"))
  expect_error(network(speed ~ ., cars, measures="T1", summary="abc"))
})

#' Post processing complexity measures
#'
#' Post-processing alternatives to deal with multiples values. This method is 
#' used by the complexity measures to summarize the obtained values.
#'
#' @param measure A list with the complexity measures values.
#' @param summary The functions to post processing the data. See the details
#'   to more information. Default: \code{c("mean", "sd")}
#' @param multiple A logical value defining if the measure should return
#'   multiple values. (Default: \code{TRUE})
#' @param ... Extra values used to the functions of summarization.
#' @details
#'  The post processing functions are used to summarize the complexity measures.
#'  They are organized into three groups: return, descriptive statistic and 
#'  distribution. Currently, the hypothesis testing post processing are not 
#'  supported.
#'
#'  In practice, there are no difference among the types, so that more than one
#'  type and functions can be combined. Usually, these function are used to
#'  summarize a set of values for each complexity measures. For instance, a 
#'  measure computed for each attribute can be summarized using the 
#'  \code{"mean"} and/or \code{"sd"}.
#'
#'  In addition to the native functions available in R, the following functions
#'  can be used:
#'  \describe{
#'    \item{"histogram"}{Computes a histogram of the given data value. The extra
#'       parameters '\code{bins}' can be used to define the number of values to
#'       be returned. The parameters '\code{max}' and '\code{min}' are used to
#'       define the range of the data. The default value for these parameters
#'       are respectively \code{10, min(x)} and \code{max(x)}.}
#'    \item{"kurtosis"}{See \code{\link[e1071]{kurtosis}}}
#'    \item{"max"}{See \code{\link{max}}}
#'    \item{"mean"}{See \code{\link{mean}}}
#'    \item{"median"}{See \code{\link{median}}}
#'    \item{"min"}{See \code{\link{min}}}
#'    \item{"quantiles"}{See \code{\link{quantile}}}
#'    \item{"sd"}{See \code{\link{sd}}}
#'    \item{"skewness"}{See \code{\link[e1071]{skewness}}}
#'    \item{"var"}{See \code{\link{var}}}
#'    \item{"return"}{Returns the original value(s) of the complexity measure.}
#'  }
#'  These functions are not restrictive, thus another functions can be applied
#'  as post-processing summarization function.
#'
#' @return A list with the post-processed complexity measures.
#'
#' @references
#'  Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
#'    data complexity library in C++. Technical Report. La Salle - Universitat 
#'    Ramon Llull.
#'
#' @examples
#' summarization(runif(15))
#' summarization(runif(15), c("min", "max"))
#' summarization(runif(15), c("quantiles", "skewness"))
#' @export
summarization <- function(measure, summary=c("mean", "sd"), multiple=TRUE, 
                          ...) {

  if(length(measure) == 0) {
    return(NA)
  }

  if(!multiple) {
    if(length(measure) > 1) {
      stop("More than one value was obtained for a single measure")
    }
    measure = as.numeric(measure[1])
    return(measure)
  }

  measure = measure[is.finite(measure)]

  res = sapply(summary, function(s) {
    do.call(s, list(measure, ...))
  }, simplify=FALSE)

  unlist(res)
}

skewness <- function(x, na.rm=FALSE, type=3, ...) {
  e1071::skewness(x, na.rm, type)
}

kurtosis <- function(x, na.rm=FALSE, type=3, ...) {
  e1071::kurtosis(x, na.rm, type)
}

quantiles <- function(x, type=1, ...) {
  tryCatch(
    stats::quantile(x, type=type, ...),
    error=function(e) stats::quantile(NA, na.rm=TRUE, ...)
  )
}

iqr <- function(x, na.rm=FALSE, ...) {
   if (!na.rm & any(is.na(x))) NA
   else stats::IQR(x, na.rm = na.rm)
}

histogram <- function(x, bins=10, min=base::min(x, na.rm=TRUE),
                 max=base::max(x, na.rm=TRUE), ...) {
  breaks <- seq(ifelse(is.finite(min), min, 0),
                ifelse(is.finite(max), max, bins), length.out=bins + 1)
  graphics::hist(as.numeric(x), breaks=breaks, plot=FALSE)$counts / length(x)
}

context("Test Summarization")

test_that("basic1.result", {

  aux = summarization(rep(1, 10), summary=c("min", "max", "mean","median"))
  expect_equal(as.numeric(aux), c(1, 1, 1, 1))
})

test_that("basic2.result", {

  aux = summarization(1:9, summary=c("min", "max", "mean","median"))
  expect_equal(as.numeric(aux), c(1, 9, 5, 5))
})

test_that("basic3.result", {

  set.seed(123)
  aux = summarization(rnorm(10000), summary=c("mean", "sd"))
  expect_equal(as.numeric(aux), c(0, 1), tolerance=0.1)
})

test_that("skewness.result", {

  set.seed(123)
  aux = summarization(runif(100), summary="skewness")
  expect_equal(as.numeric(aux), 0.03363101, tolerance=0.1)
})

test_that("kurtosis.result", {

  set.seed(123)
  aux = summarization(runif(100), summary="kurtosis")
  expect_equal(as.numeric(aux), -1.251052, tolerance=0.1)
})

test_that("quantiles.result", {

  aux = summarization(1:10, summary="quantiles")
  expect_equal(as.numeric(aux), c(1, 3, 5, 8, 10))
})

test_that("iqr.result", {

  aux = summarization(rep(1, 10), summary="iqr")
  expect_equal(as.numeric(aux), 0)

  aux = summarization(1:10, summary="iqr")
  expect_equal(as.numeric(aux), 4.5)
})

test_that("histogran.result", {

  aux = summarization(1:10, summary="histogram")
  expect_equal(as.numeric(aux), rep(0.1, 10))
})

test_that("na.result", {
  expect_equal(summarization(NULL), NA)
})

test_that("validation.error",{
  expect_error(summarization(c(1,2), multiple=FALSE))
})

#' Measures of feature correlation
#'
#' Regression task. These measures calculate the correlation of the values of 
#' the features to the outputs. If at least one feature is highly correlated to 
#' the output, this indicates that simpler functions can be fitted to the data.
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A response vector with one value for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the output column.
#' @param data A data.frame dataset contained the input and output attributes.
#' @param summary A list of summarization functions or empty for all values. See
#'  \link{summarization} method to more information. (Default: 
#'  \code{c("mean", "sd")})
#' @param ... Not used.
#' @details
#'  The following measures are allowed for this method:
#'  \describe{
#'    \item{"C1"}{Feature correlation to the output (C1) calculate the 
#'      absolute value of the Spearman correlation between each feature 
#'      and the outputs.}
#'    \item{"C2"}{Average feature correlation to the output (C2) computes the 
#'      average of the Spearman correlations of all features to the output.}
#'    \item{"C3"}{Individual feature efficiency (C3) calculates, for each 
#'      feature, the number of examples that must be removed from the dataset 
#'      until a high Spearman correlation value to the output is achieved.}
#'    \item{"C4"}{Collective feature efficiency (C4) computes the ratio of 
#'      examples removed from the dataset based on an iterative process of 
#'      linear fitting between the features and the target attribute.}
#'  }
#' @return A list named by the requested correlation measure.
#'
#' @references
#'  Ana C Lorena and Aron I Maciel and Pericles B C Miranda and Ivan G Costa and
#'    Ricardo B C Prudencio. (2018). Data complexity meta-features for 
#'    regression problems. Machine Learning, 107, 1, 209--246.
#'
#' @examples
#' ## Extract all correlation measures for regression task
#' data(cars)
#' correlation(speed ~ ., cars)
#' @export
correlation <- function(...) {
  UseMethod("correlation")
}

#' @rdname correlation
#' @export
correlation.default <- function(x, y, measures="all", summary=c("mean", "sd"), 
                                ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  if(is.factor(y)) {
    stop("label attribute needs to be numeric")
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.correlation()
  }

  measures <- match.arg(measures, ls.correlation(), TRUE)

  if (length(summary) == 0) {
    summary <- "return"
  }

  colnames(x) <- make.names(colnames(x), unique=TRUE)
  x <- normalize(binarize(x))
  y <- normalize(y)[,1]

  sapply(measures, function(f) {
    measure = eval(call(paste("r", f, sep="."), x=x, y=y))
    summarization(measure, summary, f %in% ls.correlation.multiples(), ...)
  }, simplify=FALSE)
}

#' @rdname correlation
#' @export
correlation.formula <- function(formula, data, measures="all", 
                                summary=c("mean", "sd"), ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  correlation.default(modFrame[, -1,drop=FALSE], modFrame[, 1,drop=FALSE],
    measures, summary, ...)
}

ls.correlation <- function() {
  c("C1", "C2", "C3", "C4")
}

ls.correlation.multiples <- function() {
  c("C1", "C2", "C3")
}

r.C1 <- function(x, y) {
  as.numeric(abs(stats::cor(x, method="spearman")))
}

r.C2 <- function(x, y) {
  as.numeric(abs(stats::cor(x, y, method="spearman")))
}

remove <- function(x, y, c) {

  remainingRows <- length(x)
  xorder <- rank(x)
  yorder <- rank(y)

  diff <- xorder - yorder
  correlation <- spearman(diff)

  if(correlation < 0) {
    yorder <- rank(-y)
    diff <- xorder - yorder
    correlation <- spearman(diff)
  }

  while(abs(correlation) < c && !is.na(correlation)) {

    maxPosition <- which.max(abs(diff))

    diff <- diff + ((yorder > yorder[maxPosition]) - 
      (xorder > xorder[maxPosition]))

    yorder <- yorder[-maxPosition]
    xorder <- xorder[-maxPosition]
    diff <- diff[-maxPosition]
    remainingRows <- remainingRows - 1
    correlation <- spearman(diff)

    if(is.na(correlation))
      correlation
  }

  (length(x) - remainingRows)/length(x)
}

r.C3 <- function(x, y, c=0.9) {
  as.numeric(apply(x, 2, remove, y, c))
}

r.C4 <- function(x, y, r=0.1) {

  aux <- 0
  rows <- nrow(x)

  repeat {

    aux <- aux + 1
    tmp <- stats::cor(y, x, method="spearman")
    idx <- rev(order(abs(tmp)))[1]

    model <- stats::lm(y ~ x[,idx])
    remove <- abs(model$residuals) > r

    x <- x[remove,,drop=FALSE]
    y <- y[remove]

    if(length(y) <= 1)
      return(0)

    if(sum(!remove) == length(remove) | aux == ncol(x))
      break
  }

  length(y)/rows
}

context("Test Correlation")

test_that("regression.result", {

  aux = correlation(speed ~ ., cars, measures="C2", summary="mean")
  expect_equal(as.numeric(aux$C2), 0.8303568, tolerance=0.01)

  aux = correlation(speed ~ ., cars, measures="C3", summary="min")
  expect_equal(as.numeric(aux$C3), 0.0800000)

  aux = correlation(speed ~ ., cars, measures="C4")
  expect_equal(as.numeric(aux$C4), 0.5600000)
})

test_that("validation.error",{

  expect_error(correlation(speed ~ ., cars, measures="S1"))
  expect_error(correlation(speed ~ ., cars, measures="S1", summary="abc"))

  expect_error(correlation(Species ~ ., iris))
  expect_error(correlation(Species ~ ., iris, measures="S1"))
  expect_error(correlation(Species ~ ., iris, measures="S1", summary="abc"))
})

#' Measures of linearity
#'
#' The linearity measures try to quantify if it is possible to separate the 
#' labels by a hyperplane or linear function. The underlying assumption is that 
#' a linearly separable problem can be considered simpler than a problem
#' requiring a non-linear decision boundary.
#'
#' @family complexity-measures
#' @param x A data.frame contained only the input attributes.
#' @param y A response vector with one value for each row/component of x.
#' @param measures A list of measures names or \code{"all"} to include all them.
#' @param formula A formula to define the output column.
#' @param data A data.frame dataset contained the input attributes and class.
#' @param summary A list of summarization functions or empty for all values. See
#'  \link{summarization} method to more information. (Default: 
#'  \code{c("mean", "sd")})
#' @param ... Not used.
#' @details
#'  The following classification measures are allowed for this method:
#'  \describe{
#'    \item{"L1"}{Sum of the error distance by linear programming (L1) computes 
#'      the sum of the distances of incorrectly classified examples to a linear 
#'      boundary used in their classification.}
#'    \item{"L2"}{Error rate of linear classifier (L2) computes the error rate 
#'      of the linear SVM classifier induced from dataset.}
#'    \item{"L3"}{Non-linearity of a linear classifier (L3) creates a new 
#'      dataset randomly interpolating pairs of training examples of the same 
#'      class and then induce a linear SVM on the original data and measure 
#'      the error rate in the new data points.}
#'  }
#'  The following regression measures are allowed for this method:
#'  \describe{
#'    \item{"L1"}{Mean absolute error (L1) averages the absolute values of the 
#'      residues of a multiple linear regressor.}
#'    \item{"L2"}{Residuals variance (L2) averages the square of the residuals 
#'      from a multiple linear regression.}
#'    \item{"L3"}{Non-linearity of a linear regressor (L3) measures how 
#'      sensitive the regressor is to the new randomly interpolated points.}
#'  }
#' @return A list named by the requested linearity measure.
#'
#' @references
#'  Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
#'    data complexity library in C++. Technical Report. La Salle - Universitat 
#'    Ramon Llull.
#'
#' @examples
#' ## Extract all linearity measures for classification task
#' data(iris)
#' linearity(Species ~ ., iris)
#'
#' ## Extract all linearity measures for regression task
#' data(cars)
#' linearity(speed ~ ., cars)
#' @export
linearity <- function(...) {
  UseMethod("linearity")
}

#' @rdname linearity
#' @export
linearity.default <- function(x, y, measures="all", 
                                    summary=c("mean", "sd"), ...) {

  if(!is.data.frame(x)) {
    stop("data argument must be a data.frame")
  }

  if(is.data.frame(y)) {
    y <- y[, 1]
  }

  foo <- "regression"
  if(is.factor(y)) {
    foo <- "classification"
    if(min(table(y)) < 2) {
      stop("number of examples in the minority class should be >= 2")
    }
  }

  if(nrow(x) != length(y)) {
    stop("x and y must have same number of rows")
  }

  if(measures[1] == "all") {
    measures <- ls.linearity()
  }

  measures <- match.arg(measures, ls.linearity(), TRUE)

  if (length(summary) == 0) {
    summary <- "return"
  }

  colnames(x) <- make.names(colnames(x), unique=TRUE)
  eval(call(foo, x=x, y=y, measures=measures, summary=summary))
}

#' @rdname linearity
#' @export
linearity.formula <- function(formula, data, measures="all", 
                                    summary=c("mean", "sd"), ...) {

  if(!inherits(formula, "formula")) {
    stop("method is only for formula datas")
  }

  if(!is.data.frame(data)) {
    stop("data argument must be a data.frame")
  }

  modFrame <- stats::model.frame(formula, data)
  attr(modFrame, "terms") <- NULL

  linearity.default(modFrame[, -1, drop=FALSE], modFrame[, 1, drop=FALSE],
    measures, summary, ...)
}

classification <- function(x, y, measures, summary, ...) {

  data <- data.frame(x, class=y)
  data <- ovo(data)

  model <- lapply(data, smo)
  sapply(measures, function(f) {
    measure = eval(call(paste("class", f, sep="."), model=model, data=data))
    summarization(measure, summary, f %in% ls.linearity.multiples(), ...)
  }, simplify=FALSE)
}

regression <- function(x, y, measures, summary, ...) {

  x <- normalize(x)
  y <- normalize(y)[,1]

  x <- x[order(y), ,drop=FALSE]
  y <- y[order(y)]

  model <- stats::lm(y ~ ., cbind(y=y, x))
  sapply(measures, function(f) {
    measure = eval(call(paste("regr", f, sep="."), m=model, x=x, y=y))
    summarization(measure, summary, f %in% ls.linearity.multiples(), ...)
  }, simplify=FALSE)
}

ls.linearity <- function() {
  c("L1", "L2", "L3")
}

ls.linearity.multiples <- function() {
  ls.linearity()
}

smo <- function(data) {
  e1071::svm(class ~ ., data, scale=TRUE, kernel="linear")
}

class.L1 <- function(model, data) {

  aux <- mapply(function(m, d) {
    prd <- stats::predict(m, d, decision.values=TRUE)
    err <- rownames(d[prd != d$class,])
    dst <- attr(prd, "decision.values")[err,]
    sum(abs(dst))/nrow(d)
  }, m=model, d=data)

  aux <- 1 - 1/(aux + 1)
  return(aux)
}

error <- function(pred, class) {
  1 - sum(diag(table(class, pred)))/sum(table(class, pred))
}

class.L2 <- function(model, data) {

  aux <- mapply(function(m, d) {
    prd <- stats::predict(m, d)
    error(prd, d$class)
  }, m=model, d=data)

  return(aux)
}

class.L3 <- function(model, data) {

  aux <- mapply(function(m, d) {
    tmp <- class.generate(d, nrow(d))
    prd <- stats::predict(m, tmp)
    error(prd, tmp$class)
  }, m=model, d=data)

  return(aux)
}

regr.L1 <- function(m, ...) {
  aux <- abs(m$residuals)
  aux <- 1 - 1/(aux + 1)
  return(aux)
}

regr.L2 <- function(m, ...) {
  m$residuals^2/(m$residuals^2 + 1)
}

regr.L3 <- function(m, x, y) {
  test <- regr.generate(x, y, nrow(x))
  pred <- stats::predict.lm(m, test[, -ncol(test), drop=FALSE])
  aux <- (pred - test[, ncol(test)])^2
  aux <- aux/(aux + 1)
  return(aux)
}

context("Test Linearity")

test_that("multiclass.result", {

  aux = linearity(Species ~ ., iris, measures="L1", summary="mean")
  expect_equal(as.numeric(aux$L1), 0.004335693)

  aux = linearity(Species ~ ., iris, measures="L2", summary="mean")
  expect_equal(as.numeric(aux$L2), 0.013333333)

  set.seed(123)
  aux = linearity(Species ~ ., iris, measures="L3", summary="mean")
  expect_equal(as.numeric(aux$L3), 0.01)
})

test_that("binary1.result", {

  iris = iris[1:100,]
  iris$Species = factor(iris$Species)

  aux = linearity(Species ~ ., iris, measures="L1", summary="mean")
  expect_equal(as.numeric(aux$L1), 0)

  aux = linearity(Species ~ ., iris, measures="L2", summary="mean")
  expect_equal(as.numeric(aux$L2), 0)

  set.seed(123)
  aux = linearity(Species ~ ., iris, measures="L3", summary="mean")
  expect_equal(as.numeric(aux$L3), 0)
})

test_that("binary2.result", {

  iris = iris[51:150,]
  iris$Species = factor(iris$Species)

  aux = linearity(Species ~ ., iris, measures="L1", summary="mean")
  expect_equal(as.numeric(aux$L1), 0.01300708)

  aux = linearity(Species ~ ., iris, measures="L2", summary="mean")
  expect_equal(as.numeric(aux$L2), 0.04000000)

  set.seed(123)
  aux = linearity(Species ~ ., iris, measures="L3", summary="mean")
  expect_equal(as.numeric(aux$L3), 0.02000000)
})

test_that("regression.result", {

  aux = linearity(speed ~ ., cars, measures="L1", summary="mean")
  expect_equal(as.numeric(aux$L1), 0.1020661)

  aux = linearity(speed ~ ., cars, measures="L2", summary="mean")
  expect_equal(as.numeric(aux$L2), 0.02057815)

  set.seed(123)
  aux = linearity(speed ~ ., cars, measures="L3", summary="mean")
  expect_equal(as.numeric(aux$L3), 0.01726667)
})

test_that("validation.error",{

  expect_error(linearity(Species ~ ., iris, measures="F3"))
  expect_error(linearity(Species ~ ., iris, measures="F4", summary="abc"))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{complexity}
\alias{complexity}
\alias{complexity.default}
\alias{complexity.formula}
\title{Extract the complexity measures from datasets}
\usage{
complexity(...)

\method{complexity}{default}(x, y, groups = "all", summary = c("mean", "sd"), ...)

\method{complexity}{formula}(formula, data, groups = "all", summary = c("mean", "sd"), ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A response vector with one value for each row/component of x.}

\item{groups}{A list of complexity measures groups or \code{"all"} to include
all of them.}

\item{summary}{A list of summarization functions or empty for all values. See
\link{summarization} method to more information. (Default: 
\code{c("mean", "sd")})}

\item{formula}{A formula to define the output column.}

\item{data}{A data.frame dataset contained the input and output attributes.}
}
\value{
A numeric vector named by the requested complexity measures.
}
\description{
This function is responsable to extract the complexity measures from the 
classification and regression tasks. For such, they take into account the 
overlap between classes imposed by feature values, the separability and 
distribution of the data points and the value of structural measures based on
the representation of the dataset as a graph structure. To set specific 
parameters for each group, use the characterization function.
}
\details{
The following groups are allowed for this method:
 \describe{
   \item{"featurebased"}{The feature feature-based measures characterize how 
     informative the available features are to separate the classes See 
     \link{featurebased} for more details.}
   \item{"neighborhood"}{Neighborhood measures characterize the presence and 
     density of same or different classes in local neighborhoods. See 
     \link{neighborhood} for more details.}
   \item{"linearity"}{Linearity measures try to quantify whether the labels 
     can be linearly separated. See \link{linearity} for more details.}
   \item{"dimensionality"}{The dimensionality measures compute information on
     how smoothly the examples are distributed within the attributes. See 
     \link{dimensionality} for more details.}
   \item{"balance"}{Class balance measures take into account the numbers of 
     examples per class in the dataset. See \link{balance} for more details.}
   \item{"network"}{Network measures represent the dataset as a graph and 
     extract structural information from it. See \link{network} for more 
     details.}
   \item{"correlation"}{Capture the relationship of the feature values with 
     the outputs. See \link{correlation} for more details.}
   \item{"smoothness"}{Estimate the smoothness of the function that must be 
     fitted to the data. See \link{smoothness} for more details.}
 }
}
\examples{
## Extract all complexity measures for classification task
data(iris)
complexity(Species ~ ., iris)

## Extract all complexity measures for regression task
data(cars)
complexity(speed ~ ., cars)
}
\references{
Tin K Ho and Mitra Basu. (2002). Complexity measures of supervised 
   classification problems. IEEE Transactions on Pattern Analysis and Machine
   Intelligence, 24, 3, 289--300.

 Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for 
   the data complexity library in C++. Technical Report. La Salle - 
   Universitat Ramon Llull.

 Ana C Lorena and Aron I Maciel and Pericles B C Miranda and Ivan G Costa and
   Ricardo B C Prudencio. (2018). Data complexity meta-features for 
   regression problems. Machine Learning, 107, 1, 209--246.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature-based.R
\name{featurebased}
\alias{featurebased}
\alias{featurebased.default}
\alias{featurebased.formula}
\title{Measures of feature-based}
\usage{
featurebased(...)

\method{featurebased}{default}(x, y, measures = "all", summary = c("mean", "sd"), ...)

\method{featurebased}{formula}(formula, data, measures = "all", summary = c("mean", "sd"), ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A factor response vector with one label for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{summary}{A list of summarization functions or empty for all values. See
\link{summarization} method to more information. (Default: 
\code{c("mean", "sd")})}

\item{formula}{A formula to define the class column.}

\item{data}{A data.frame dataset contained the input attributes and class.}
}
\value{
A list named by the requested feature-based measure.
}
\description{
Classification task. The feature-based measures evaluate how informative the 
available features are to separate the classes. If there is at least one very
discriminative feature in the dataset, the problem can be considered simpler 
than if there is no such an attribute.
}
\details{
The following measures are allowed for this method:
 \describe{
   \item{"F1"}{Maximum Fisher's Discriminant Ratio (F1) measures the overlap 
     between the values of the features and takes the value of the largest 
     discriminant ratio among all the available features.}
   \item{"F1v"}{Directional-vector maximum Fisher's discriminant ratio (F1v)
     complements F1 by searching for a vector able to separate two classes 
     after the training examples have been projected into it.}
   \item{"F2"}{Volume of the overlapping region (F2) computes the overlap of 
     the distributions of the features values within the classes. F2 can be 
     determined by finding, for each feature its minimum and maximum values 
     in the classes.}
   \item{"F3"}{The maximum individual feature efficiency (F3) of each 
     feature is given by the ratio between the number of examples that are 
     not in the overlapping region of two classes and the total number of 
     examples. This measure returns the maximum of the values found among 
     the input features.}
   \item{"F4"}{Collective feature efficiency (F4) get an overview on how 
     various features may work together in data separation. First the most 
     discriminative feature according to F3 is selected and all examples that
     can be separated by this feature are removed from the dataset. The 
     previous step is repeated on the remaining dataset until all the 
     features have been considered or no example remains. F4 returns the 
     ratio of examples that have been discriminated.}
 }
}
\examples{
## Extract all feature-based measures for classification task
data(iris)
featurebased(Species ~ ., iris)
}
\references{
Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
   data complexity library in C++. Technical Report. La Salle - Universitat
   Ramon Llull.
}
\seealso{
Other complexity-measures:
\code{\link[=balance]{balance()}},
\code{\link[=correlation]{correlation()}},
\code{\link[=dimensionality]{dimensionality()}},
\code{\link[=linearity]{linearity()}},
\code{\link[=neighborhood]{neighborhood()}},
\code{\link[=network]{network()}},
\code{\link[=smoothness]{smoothness()}}
}
\concept{complexity-measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/balance.R
\name{balance}
\alias{balance}
\alias{balance.default}
\alias{balance.formula}
\title{Measures of class balance}
\usage{
balance(...)

\method{balance}{default}(x, y, measures = "all", ...)

\method{balance}{formula}(formula, data, measures = "all", ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A factor response vector with one label for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{formula}{A formula to define the class column.}

\item{data}{A data.frame dataset contained the input attributes and class.}
}
\value{
A list named by the requested class balance measure.
}
\description{
Classification task. These measures capture the differences in the number of 
examples per class in the dataset. When these differences are severe, 
problems related to generalization of the ML classification techniques could 
happen because of the imbalance ratio.
}
\details{
The following measures are allowed for this method:
 \describe{
   \item{"B1"}{The entropy of class proportions (B1) capture the imbalance in
     a dataset based on the proportions of examples per class.}
   \item{"B2"}{The imbalance ratio (B2) is an index computed for measuring
     class balance. This is a version of the measure that is also suited for 
     multiclass classification problems.}
 }
}
\examples{
## Extract all balance measures for classification task
data(iris)
balance(Species ~ ., iris)
}
\references{
Ana C Lorena, Ivan G Costa, Newton Spolaor and Marcilio C P Souto. (2012). 
   Analysis of complexity indices for classification problems: Cancer gene 
   expression data. Neurocomputing 75, 1, 33--42.

 Ajay K Tanwani and Muddassar Farooq. (2010). Classification potential vs. 
   classification accuracy: a comprehensive study of evolutionary algorithms 
   with biomedical datasets. Learning Classifier Systems 6471, 127--144.
}
\seealso{
Other complexity-measures:
\code{\link[=correlation]{correlation()}},
\code{\link[=dimensionality]{dimensionality()}},
\code{\link[=featurebased]{featurebased()}},
\code{\link[=linearity]{linearity()}},
\code{\link[=neighborhood]{neighborhood()}},
\code{\link[=network]{network()}},
\code{\link[=smoothness]{smoothness()}}
}
\concept{complexity-measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smoothness.R
\name{smoothness}
\alias{smoothness}
\alias{smoothness.default}
\alias{smoothness.formula}
\title{Measures of smoothness}
\usage{
smoothness(...)

\method{smoothness}{default}(x, y, measures = "all", summary = c("mean", "sd"), ...)

\method{smoothness}{formula}(formula, data, measures = "all", summary = c("mean", "sd"), ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A response vector with one value for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{summary}{A list of summarization functions or empty for all values. See
\link{summarization} method to more information. (Default: 
\code{c("mean", "sd")})}

\item{formula}{A formula to define the output column.}

\item{data}{A data.frame dataset contained the input and output attributes.}
}
\value{
A list named by the requested smoothness measure.
}
\description{
Regression task. In regression problems, the smoother the function to be 
fitted to the data, the simpler it shall be. Larger variations in the inputs
and/or outputs, on the other hand, usually indicate the existence of more 
intricate relationships between them.
}
\details{
The following measures are allowed for this method:
 \describe{
   \item{"S1"}{Output distribution (S1) monitors whether the examples 
     joined in the MST have similar output values. Lower values indicate 
     simpler problems, where the outputs of similar examples in the input 
     space are also next to each other.}
   \item{"S2"}{Input distribution (S2) measure how similar in the input space
     are data items with similar outputs based on distance.}
   \item{"S3"}{Error of a nearest neighbor regressor (S3) calculates the mean
     squared error of a 1-nearest neighbor regressor  using leave-one-out.}
   \item{"S4"}{Non-linearity of nearest neighbor regressor (S4) calculates 
     the mean squared error of a 1-nearest neighbor regressor to the new 
     randomly interpolated points.}
 }
}
\examples{
## Extract all smoothness measures for regression task
data(cars)
smoothness(speed ~ ., cars)
}
\references{
Ana C Lorena and Aron I Maciel and Pericles B C Miranda and Ivan G Costa and
   Ricardo B C Prudencio. (2018). Data complexity meta-features for 
   regression problems. Machine Learning, 107, 1, 209--246.
}
\seealso{
Other complexity-measures:
\code{\link[=balance]{balance()}},
\code{\link[=correlation]{correlation()}},
\code{\link[=dimensionality]{dimensionality()}},
\code{\link[=featurebased]{featurebased()}},
\code{\link[=linearity]{linearity()}},
\code{\link[=neighborhood]{neighborhood()}},
\code{\link[=network]{network()}}
}
\concept{complexity-measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlation.R
\name{correlation}
\alias{correlation}
\alias{correlation.default}
\alias{correlation.formula}
\title{Measures of feature correlation}
\usage{
correlation(...)

\method{correlation}{default}(x, y, measures = "all", summary = c("mean", "sd"), ...)

\method{correlation}{formula}(formula, data, measures = "all", summary = c("mean", "sd"), ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A response vector with one value for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{summary}{A list of summarization functions or empty for all values. See
\link{summarization} method to more information. (Default: 
\code{c("mean", "sd")})}

\item{formula}{A formula to define the output column.}

\item{data}{A data.frame dataset contained the input and output attributes.}
}
\value{
A list named by the requested correlation measure.
}
\description{
Regression task. These measures calculate the correlation of the values of 
the features to the outputs. If at least one feature is highly correlated to 
the output, this indicates that simpler functions can be fitted to the data.
}
\details{
The following measures are allowed for this method:
 \describe{
   \item{"C1"}{Feature correlation to the output (C1) calculate the 
     absolute value of the Spearman correlation between each feature 
     and the outputs.}
   \item{"C2"}{Average feature correlation to the output (C2) computes the 
     average of the Spearman correlations of all features to the output.}
   \item{"C3"}{Individual feature efficiency (C3) calculates, for each 
     feature, the number of examples that must be removed from the dataset 
     until a high Spearman correlation value to the output is achieved.}
   \item{"C4"}{Collective feature efficiency (C4) computes the ratio of 
     examples removed from the dataset based on an iterative process of 
     linear fitting between the features and the target attribute.}
 }
}
\examples{
## Extract all correlation measures for regression task
data(cars)
correlation(speed ~ ., cars)
}
\references{
Ana C Lorena and Aron I Maciel and Pericles B C Miranda and Ivan G Costa and
   Ricardo B C Prudencio. (2018). Data complexity meta-features for 
   regression problems. Machine Learning, 107, 1, 209--246.
}
\seealso{
Other complexity-measures:
\code{\link[=balance]{balance()}},
\code{\link[=dimensionality]{dimensionality()}},
\code{\link[=featurebased]{featurebased()}},
\code{\link[=linearity]{linearity()}},
\code{\link[=neighborhood]{neighborhood()}},
\code{\link[=network]{network()}},
\code{\link[=smoothness]{smoothness()}}
}
\concept{complexity-measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linearity.R
\name{linearity}
\alias{linearity}
\alias{linearity.default}
\alias{linearity.formula}
\title{Measures of linearity}
\usage{
linearity(...)

\method{linearity}{default}(x, y, measures = "all", summary = c("mean", "sd"), ...)

\method{linearity}{formula}(formula, data, measures = "all", summary = c("mean", "sd"), ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A response vector with one value for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{summary}{A list of summarization functions or empty for all values. See
\link{summarization} method to more information. (Default: 
\code{c("mean", "sd")})}

\item{formula}{A formula to define the output column.}

\item{data}{A data.frame dataset contained the input attributes and class.}
}
\value{
A list named by the requested linearity measure.
}
\description{
The linearity measures try to quantify if it is possible to separate the 
labels by a hyperplane or linear function. The underlying assumption is that 
a linearly separable problem can be considered simpler than a problem
requiring a non-linear decision boundary.
}
\details{
The following classification measures are allowed for this method:
 \describe{
   \item{"L1"}{Sum of the error distance by linear programming (L1) computes 
     the sum of the distances of incorrectly classified examples to a linear 
     boundary used in their classification.}
   \item{"L2"}{Error rate of linear classifier (L2) computes the error rate 
     of the linear SVM classifier induced from dataset.}
   \item{"L3"}{Non-linearity of a linear classifier (L3) creates a new 
     dataset randomly interpolating pairs of training examples of the same 
     class and then induce a linear SVM on the original data and measure 
     the error rate in the new data points.}
 }
 The following regression measures are allowed for this method:
 \describe{
   \item{"L1"}{Mean absolute error (L1) averages the absolute values of the 
     residues of a multiple linear regressor.}
   \item{"L2"}{Residuals variance (L2) averages the square of the residuals 
     from a multiple linear regression.}
   \item{"L3"}{Non-linearity of a linear regressor (L3) measures how 
     sensitive the regressor is to the new randomly interpolated points.}
 }
}
\examples{
## Extract all linearity measures for classification task
data(iris)
linearity(Species ~ ., iris)

## Extract all linearity measures for regression task
data(cars)
linearity(speed ~ ., cars)
}
\references{
Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
   data complexity library in C++. Technical Report. La Salle - Universitat 
   Ramon Llull.
}
\seealso{
Other complexity-measures:
\code{\link[=balance]{balance()}},
\code{\link[=correlation]{correlation()}},
\code{\link[=dimensionality]{dimensionality()}},
\code{\link[=featurebased]{featurebased()}},
\code{\link[=neighborhood]{neighborhood()}},
\code{\link[=network]{network()}},
\code{\link[=smoothness]{smoothness()}}
}
\concept{complexity-measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network.R
\name{network}
\alias{network}
\alias{network.default}
\alias{network.formula}
\title{Measures of network}
\usage{
network(...)

\method{network}{default}(x, y, measures = "all", eps = 0.15, summary = c("mean", "sd"), ...)

\method{network}{formula}(
  formula,
  data,
  measures = "all",
  eps = 0.15,
  summary = c("mean", "sd"),
  ...
)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A factor response vector with one label for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{eps}{The percentage of nodes in the graph to be connected.}

\item{summary}{A list of summarization functions or empty for all values. See
\link{summarization} method to more information. (Default: 
\code{c("mean", "sd")})}

\item{formula}{A formula to define the class column.}

\item{data}{A data.frame dataset contained the input attributes and class.}
}
\value{
A list named by the requested network measure.
}
\description{
Classification task. The network measures represent the dataset as a graph 
and extract structural information from it. The transformation between raw 
data and the graph representation is based on the epsilon-NN algorithm. Next,
a post-processing step is applied to the graph, pruning edges between 
examples of opposite classes.
}
\details{
The following measures are allowed for this method:
 \describe{
   \item{"G1"}{Average Density of the network (G1) represents the 
     number of edges in the graph, divided by the maximum number of edges 
     between pairs of data points.}
   \item{"G2"}{Clustering coefficient (G2) averages the clustering 
     tendency of the vertexes by the ratio of existent edges between its 
     neighbors and the total number of edges that could possibly exist 
     between them.}
   \item{"G3"}{Hubs score (G3) is given by the number of connections it  
     has to other nodes, weighted by the number of connections these 
     neighbors have.}
 }
}
\examples{
## Extract all network measures for classification task
data(iris)
network(Species ~ ., iris)
}
\references{
Gleison Morais and Ronaldo C Prati. (2013). Complex Network Measures for 
   Data Set Characterization. In 2nd Brazilian Conference on Intelligent 
   Systems (BRACIS). 12--18.

 Luis P F Garcia, Andre C P L F de Carvalho and Ana C Lorena. (2015). Effect
   of label noise in the complexity of classification problems. 
   Neurocomputing 160, 108--119.
}
\seealso{
Other complexity-measures:
\code{\link[=balance]{balance()}},
\code{\link[=correlation]{correlation()}},
\code{\link[=dimensionality]{dimensionality()}},
\code{\link[=featurebased]{featurebased()}},
\code{\link[=linearity]{linearity()}},
\code{\link[=neighborhood]{neighborhood()}},
\code{\link[=smoothness]{smoothness()}}
}
\concept{complexity-measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarization.R
\name{summarization}
\alias{summarization}
\title{Post processing complexity measures}
\usage{
summarization(measure, summary = c("mean", "sd"), multiple = TRUE, ...)
}
\arguments{
\item{measure}{A list with the complexity measures values.}

\item{summary}{The functions to post processing the data. See the details
to more information. Default: \code{c("mean", "sd")}}

\item{multiple}{A logical value defining if the measure should return
multiple values. (Default: \code{TRUE})}

\item{...}{Extra values used to the functions of summarization.}
}
\value{
A list with the post-processed complexity measures.
}
\description{
Post-processing alternatives to deal with multiples values. This method is 
used by the complexity measures to summarize the obtained values.
}
\details{
The post processing functions are used to summarize the complexity measures.
 They are organized into three groups: return, descriptive statistic and 
 distribution. Currently, the hypothesis testing post processing are not 
 supported.

 In practice, there are no difference among the types, so that more than one
 type and functions can be combined. Usually, these function are used to
 summarize a set of values for each complexity measures. For instance, a 
 measure computed for each attribute can be summarized using the 
 \code{"mean"} and/or \code{"sd"}.

 In addition to the native functions available in R, the following functions
 can be used:
 \describe{
   \item{"histogram"}{Computes a histogram of the given data value. The extra
      parameters '\code{bins}' can be used to define the number of values to
      be returned. The parameters '\code{max}' and '\code{min}' are used to
      define the range of the data. The default value for these parameters
      are respectively \code{10, min(x)} and \code{max(x)}.}
   \item{"kurtosis"}{See \code{\link[e1071]{kurtosis}}}
   \item{"max"}{See \code{\link{max}}}
   \item{"mean"}{See \code{\link{mean}}}
   \item{"median"}{See \code{\link{median}}}
   \item{"min"}{See \code{\link{min}}}
   \item{"quantiles"}{See \code{\link{quantile}}}
   \item{"sd"}{See \code{\link{sd}}}
   \item{"skewness"}{See \code{\link[e1071]{skewness}}}
   \item{"var"}{See \code{\link{var}}}
   \item{"return"}{Returns the original value(s) of the complexity measure.}
 }
 These functions are not restrictive, thus another functions can be applied
 as post-processing summarization function.
}
\examples{
summarization(runif(15))
summarization(runif(15), c("min", "max"))
summarization(runif(15), c("quantiles", "skewness"))
}
\references{
Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
   data complexity library in C++. Technical Report. La Salle - Universitat 
   Ramon Llull.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neighborhood.R
\name{neighborhood}
\alias{neighborhood}
\alias{neighborhood.default}
\alias{neighborhood.formula}
\title{Measures of neighborhood}
\usage{
neighborhood(...)

\method{neighborhood}{default}(x, y, measures = "all", summary = c("mean", "sd"), ...)

\method{neighborhood}{formula}(formula, data, measures = "all", summary = c("mean", "sd"), ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A factor response vector with one label for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{summary}{A list of summarization functions or empty for all values. See
\link{summarization} method to more information. (Default: 
\code{c("mean", "sd")})}

\item{formula}{A formula to define the class column.}

\item{data}{A data.frame dataset contained the input attributes and class.}
}
\value{
A list named by the requested neighborhood measure.
}
\description{
Classification task. The Neighborhood measures analyze the neighborhoods of 
the data items and try to capture class overlapping and the shape of the 
decision boundary. They work over a distance matrix storing the distances 
between all pairs of data points in the dataset.
}
\details{
The following measures are allowed for this method:
 \describe{
   \item{"N1"}{Fraction of borderline points (N1) computes the percentage of 
     vertexes incident to edges connecting examples of opposite classes in 
     a Minimum Spanning Tree (MST).}
   \item{"N2"}{Ratio of intra/extra class nearest neighbor distance (N2)  
     computes the ratio of two sums: intra-class and inter-class. The former 
     corresponds to the sum of the distances between each example and its 
     closest neighbor from the same class. The later is the sum of the 
     distances between each example and its closest neighbor from another 
     class (nearest enemy).}
   \item{"N3"}{Error rate of the nearest neighbor (N3) classifier corresponds
     to the error rate of a one Nearest Neighbor (1NN) classifier, estimated 
     using a leave-one-out procedure in dataset.}
   \item{"N4"}{Non-linearity of the nearest neighbor classifier (N4) creates 
     a new dataset randomly interpolating pairs of training examples of the 
     same class and then induce a the 1NN classifier on the original data and
     measure the error rate in the new data points.}
   \item{"N5"}{Fraction of hyperspheres covering data (N5) builds 
     hyperspheres centered at each one of the training examples, which have 
     their radios growth until the hypersphere reaches an example of another 
     class. Afterwards, smaller hyperspheres contained in larger hyperspheres 
     are eliminated. N5 is finally defined as the ratio between the number of 
     the remaining hyperspheres and the total number of examples in the 
     dataset.}
   \item{"N6"}{Local Set Average Cardinality (N6) is based on Local Set 
     (LS) and defined as the set of points from the dataset whose distance of
     each example is smaller than the distance from the exemples of the 
     different class. LSC is the average of the LS.}
 }
}
\examples{
## Extract all neighborhood measures for classification task
data(iris)
neighborhood(Species ~ ., iris)
}
\references{
Albert Orriols-Puig, Nuria Macia and Tin K Ho. (2010). Documentation for the
   data complexity library in C++. Technical Report. La Salle - Universitat 
   Ramon Llull.

 Enrique Leyva, Antonio Gonzalez and Raul Perez. (2014). A Set of Complexity
   Measures Designed for Applying Meta-Learning to Instance Selection. IEEE
   Transactions on Knowledge and Data Engineering 27, 2, 354--367.
}
\seealso{
Other complexity-measures:
\code{\link[=balance]{balance()}},
\code{\link[=correlation]{correlation()}},
\code{\link[=dimensionality]{dimensionality()}},
\code{\link[=featurebased]{featurebased()}},
\code{\link[=linearity]{linearity()}},
\code{\link[=network]{network()}},
\code{\link[=smoothness]{smoothness()}}
}
\concept{complexity-measures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dimensionality.R
\name{dimensionality}
\alias{dimensionality}
\alias{dimensionality.default}
\alias{dimensionality.formula}
\title{Measures of dimensionality}
\usage{
dimensionality(...)

\method{dimensionality}{default}(x, y, measures = "all", ...)

\method{dimensionality}{formula}(formula, data, measures = "all", ...)
}
\arguments{
\item{...}{Not used.}

\item{x}{A data.frame contained only the input attributes.}

\item{y}{A response vector with one value for each row/component of x.}

\item{measures}{A list of measures names or \code{"all"} to include all them.}

\item{formula}{A formula to define the output column.}

\item{data}{A data.frame dataset contained the input and output attributes.}
}
\value{
A list named by the requested dimensionality measure.
}
\description{
These measures give an indicative of data sparsity. They capture how sparse 
a datasets tend to have regions of low density. These regions are know to be 
more difficult to extract good classification and regression models.
}
\details{
The following measures are allowed for this method:
 \describe{
   \item{"D1"}{Average number of points per dimension (D1) is given by the 
     ratio between the number of examples and dimensionality of the dataset.}
   \item{"D2"}{Average number of points per PCA (D2) is similar to T2, but 
     uses the number of PCA components needed to represent 95% of data 
     variability as the base of data sparsity assessment.}
   \item{"D3"}{Ratio of the PCA Dimension to the Original (D3) estimates the
     proportion of relevant and the original dimensions for a dataset.}
 }
}
\examples{
## Extract all dimensionality measures for classification task
data(iris)
dimensionality(Species ~ ., iris)

## Extract all dimensionality measures for regression task
data(cars)
dimensionality(speed ~ ., cars)
}
\references{
Ana C Lorena, Ivan G Costa, Newton Spolaor and Marcilio C P Souto. (2012). 
   Analysis of complexity indices for classification problems: Cancer gene 
   expression data. Neurocomputing 75, 1, 33--42.
}
\seealso{
Other complexity-measures:
\code{\link[=balance]{balance()}},
\code{\link[=correlation]{correlation()}},
\code{\link[=featurebased]{featurebased()}},
\code{\link[=linearity]{linearity()}},
\code{\link[=neighborhood]{neighborhood()}},
\code{\link[=network]{network()}},
\code{\link[=smoothness]{smoothness()}}
}
\concept{complexity-measures}
